@@ -20,6 +20,7 @@ import aptdaemon.client as aptclient
 import gettext
 import os
 
+from aptdaemon.enums import EXIT_SUCCESS
 from gi.repository import Gtk, GObject, GLib, Gdk, GdkPixbuf, Pango
 from ubuntuone.installer import (CLIENT_SHELL_PACKAGE,
                                  CONTROL_PANEL_COMMAND,
@@ -109,12 +110,12 @@ class Window(Gtk.Window):
         self.set_position(Gtk.WindowPosition.CENTER)
 
         vbox = Gtk.VBox()
+        vbox.set_border_width(12)
         vbox.set_spacing(24)
         self.add(vbox)
         vbox.show()
 
         self.__header = Gtk.HBox()
-        self.__header.set_border_width(12)
         self.__header.set_spacing(64)
         vbox.pack_start(self.__header, False, False, 0)
         self.__header.show()
@@ -142,7 +143,7 @@ class Window(Gtk.Window):
         self.__header.pack_end(self.__hlogo, False, False, 0)
         self.__hlogo.show()
 
-        logo = self.__find_data_file('u1logo.png')
+        logo = self.__find_data_file('u1logo.svg')
         if logo:
             self.__hlogo.set_from_file(logo)
 
@@ -163,11 +164,19 @@ class Window(Gtk.Window):
         self.__notebook.append_page(self.__progress_page, None)
         self.__progress_page.show()
 
+        # Label widget for error text
+        self.__err_label = Gtk.Label('')
+        self.__err_label.set_ellipsize(Pango.EllipsizeMode.MIDDLE)
+        self.__err_label.set_line_wrap(True)
+
+        self.__error_page = self.__construct_error_page()
+        self.__notebook.append_page(self.__error_page, None)
+        self.__error_page.show()
+
         vbox.pack_start(self.__notebook, False, False, 0)
         self.__notebook.show()
 
         self.__action_area = Gtk.HButtonBox()
-        self.__action_area.set_border_width(12)
         self.__action_area.set_spacing(12)
         self.__action_area.set_layout(Gtk.ButtonBoxStyle.END)
         vbox.pack_end(self.__action_area, False, True, 0)
@@ -181,13 +190,20 @@ class Window(Gtk.Window):
 
         self.__main_button = Gtk.Button.new_with_mnemonic(_(u'I_nstall'))
         self.__main_button.set_image(Gtk.Image.new_from_stock(
-                Gtk.STOCK_OK, Gtk.IconSize.BUTTON))
+                Gtk.STOCK_EXECUTE, Gtk.IconSize.BUTTON))
         self.__main_button.connect('clicked', lambda x: self.emit(
                 'response', Gtk.ResponseType.OK))
         self.__action_area.add(self.__main_button)
         self.__main_button.grab_focus()
         self.__main_button.show()
 
+        # Button for error conditions
+        self.__close_button = Gtk.Button.new_from_stock(Gtk.STOCK_CLOSE)
+        self.__close_button.connect('clicked', lambda x: self.emit(
+                'response', Gtk.ResponseType.CANCEL))
+        self.__action_area.add(self.__close_button)
+
+        # Button for the external link
         self.__lm_button = Gtk.LinkButton.new_with_label(
             'https://one.ubuntu.com/', _(u'Learn more'))
         self.__action_area.add(self.__lm_button)
@@ -246,7 +262,6 @@ class Window(Gtk.Window):
     def __construct_info_page(self):
         """Build the initial info page."""
         page = Gtk.HBox()
-        page.set_border_width(6)
         page.set_spacing(12)
         page.show()
 
@@ -259,7 +274,7 @@ class Window(Gtk.Window):
         table = Gtk.Table(5, 2, False)
         table.set_row_spacings(6)
         table.set_col_spacings(12)
-        page.pack_start(table, True, True, 24)
+        page.pack_start(table, True, True, 0)
         table.show()
 
         image = Gtk.Image()
@@ -322,7 +337,6 @@ class Window(Gtk.Window):
     def __construct_progress_page(self):
         """Build the install progress page."""
         page = Gtk.VBox()
-        page.set_border_width(24)
         page.set_spacing(6)
         page.show()
 
@@ -336,42 +350,44 @@ class Window(Gtk.Window):
 
         return page
 
-    def __get_series(self):
-        """Get the series we're running on."""
-        on_ubuntu = False
-        series = None
+    def __construct_error_page(self):
+        """A page for showing errors to the user."""
+        page = Gtk.HBox()
+        page.set_spacing(12)
+        page.show()
 
-        def get_value(keypair):
-            return keypair.split('=')[1].strip()
+        image = Gtk.Image.new_from_stock(Gtk.STOCK_DIALOG_ERROR,
+                                         Gtk.IconSize.DIALOG)
+        image.set_alignment(0.5, 0.0)
+        page.pack_start(image, False, False, 0)
+        image.show()
 
-        try:
-            with open('/etc/lsb-release', 'r') as f:
-                for line in f.readlines():
-                    if line.startswith('DISTRIB_ID'):
-                        on_ubuntu = get_value(line) == u'Ubuntu'
-                    if line.startswith('DISTRIB_CODENAME'):
-                        series = get_value(line)
-            if not on_ubuntu or series is None:
-                raise UnsupportedDistribution(
-                    'This distribution is not supported by Ubuntu One.')
-        except (OSError, IOError, UnsupportedDistribution), error:
-            self.__got_error(error)
+        # Align and pack the error label
+        self.__err_label.set_alignment(0.0, 0.0)
+        page.pack_start(self.__err_label, True, True, 0)
+        self.__err_label.show()
 
-        return series
+        return page
 
     def __got_error(self, error):
         """Got an error trying to set up Ubuntu One."""
-        print error
-        Gtk.main_quit()
+        self.__cancel_button.hide()
+        self.__main_button.show()
+        self.__close_button.show()
+        self.__err_label.set_text(error)
+        self.__notebook.set_current_page(2)
 
     @inline_callbacks
     def __install_u1(self, *args, **kwargs):
         """Install the packages."""
         self.__apt_progress.set_fraction(0.0)
 
-        def finished(*args, **kwargs):
-            GLib.spawn_command_line_async(CONTROL_PANEL_COMMAND)
-            Gtk.main_quit()
+        def finished(transaction, exit_status, *args, **kwargs):
+            if exit_status != EXIT_SUCCESS:
+                self.__got_error(transaction.error_details)
+            else:
+                GLib.spawn_command_line_async(CONTROL_PANEL_COMMAND)
+                Gtk.main_quit()
 
         transaction = yield self.client.install_packages(
             package_names=self.__get_package_list())
@@ -383,23 +399,15 @@ class Window(Gtk.Window):
     def __update_cache(self, *args, **kwargs):
         """Update the cache."""
         self.__apt_progress.set_fraction(0.0)
-        transaction = yield self.client.update_cache(
-            sources_list='ubuntuone-stable-ppa.list')
-        transaction.connect('finished', self.__install_u1)
-        self.__apt_progress.set_transaction(transaction)
-        transaction.run()
 
-    @inline_callbacks
-    def __add_stable_ppa(self):
-        """Add the Ubuntu One 'stable' PPA to apt."""
-        transaction = yield self.client.add_repository(
-            src_type='deb',
-            uri='http://ppa.launchpad.net/ubuntuone/stable/ubuntu',
-            dist=self.__get_series(),
-            comps=['main'],
-            comment='added by Ubuntu One installer',
-            sourcesfile='ubuntuone-stable-ppa.list')
-        transaction.connect('finished', self.__update_cache)
+        def finished(transaction, exit_status, *args, **kwargs):
+            if exit_status != EXIT_SUCCESS:
+                self.__got_error(transaction.error_details)
+            else:
+                self.__install_u1()
+
+        transaction = yield self.client.update_cache()
+        transaction.connect('finished', finished)
         self.__apt_progress.set_transaction(transaction)
         transaction.run()
 
@@ -408,6 +416,7 @@ class Window(Gtk.Window):
         # Hide the buttons
         self.__cancel_button.hide()
         self.__main_button.hide()
+        self.__close_button.hide()
 
         # Switch to the progress page
         self.__notebook.set_current_page(1)
